# Freva Metadata Ingestion

This project will ingest metadata retrieved from the filesystem per DRS specifications and store them in a Solr instance
for use by Freva

# Configuration

This expects 2 config files in the project directory for this project as defined by the [directories
crate](https://docs.rs/directories/latest/directories/). A table for each OS is
[here](https://docs.rs/directories/latest/directories/struct.ProjectDirs.html#method.config_dir), for Linux users it
will be `$HOME/.config/freva`. Within the directory it will look for `drs_conf.toml` and `evaluation_system.conf`.
Examples of the config files are available in `./.docker`.

This can also be overridden with a `EVALUATION_SYSTEM_CONFIG_DIR` envvar.

A small example config (reformatted version of the file found in `.docker/drs_config.toml`):

```toml
[cmip5_name]
root_dir = ".docker/data/cmip5"
drs_format = "cmip5"

[observations]
root_dir = ".docker/data/obs"
drs_format = "custom"
parts_dir = [
    "project", "product", "institute", "model", "experiment", "time_frequency", "realm",
    "cmor_table", "ensemble", "version", "variable"
]
parts_file_name = ["variable", "time_frequency", "experiment", "level", "version", "time"]
parts_time = "start_time-end_time"
[observations.defaults]
project = "observations"

```

## Custom Dataset Definitions

Custom dataset definitions require the additional values `parts_dir` and `parts_file_name`, `parts_time` and `defaults`
are both optional.

- `parts_dir` defines which sections of the path (`/` separated) corresponds to which metadata values,
  note the lack of a `filename` element at the end which is assumed to be there in a valid DRS path.
- `parts_file_name` defines the same for the file name (`_` separated) and generally assumes that the file ends with
  `.nc` as per the DRS specifications.
- `defaults` is used to define constant values for missing elements. For example if you have a `cmip5` structure but
  without the `cmip5` that should be at the start of the path, you could create a custom structure and set
  `project=cmip5` in `defaults`.
- `parts_time` is currently unused and comes from the previous version of our ingestion code. It defaults to `start-end`
  if not set.

The required keys are `project`, `product`, `institute`, `model`, `experiment`, `time_frequency`, `realm`, `cmor_table`,
`ensemble`, and `variable`. `version` is used but optional, if it's missing the ingested files will simply not have a
version value. All other values will be ignored which can be useful to document what an unused element is. Values can
appear multiple times and in such cases the value that is ultimately used is the last instance found in the order of
`defaults`, `parts_dir`, `parts_file_name` (and within each, the last instance will take priority).

For example, given a (not quite valid) config like:

```toml
[foo]
root_dir = "..."
drs_format = "custom"
parts_dir = [
    "project", "product", "variable", "model", "time_frequency", "realm", "variable"
]
parts_file_name = ["variable", "time_frequency", "experiment", "variable", "version"]
[foo.defaults]
variable = "variable0"
```

Note that `variable` is there five times, first in `defaults`, twice in `parts_dirs`, and twice in `parts_file_name`.

Given a path `proj/prod/variable1/mod/freq/realm/variable2/variable3_freq_exp_variable4_v1.nc`, the value for
`variable` that will be ingested is `variable4` as it is the last one found.

# Installation

Downloads are available on the [Releases page](https://gitlab.dkrz.de/freva/freva-ingest/-/releases). Download the
appropriate binary from the latest release for your OS and architecture (currently only x86_64 Linux is built
automatically), unzip the download, and place the binary in your `PATH`.
