use anyhow::{Context, Error};
use freva::drs::Config as DrsConfig;
use freva::solr::Solr;

use crate::cli::IngestOpts;
use crate::config::Config;

pub async fn ingest(opts: &IngestOpts, conf: &Config, drs_conf: &DrsConfig) -> Result<(), Error> {
    let solr = Solr::new(conf.solr.host.clone(), conf.solr.port, None)?;

    let reports = freva::drs::ingest(
        &solr,
        drs_conf,
        &opts.data_dir.as_deref(),
        opts.batch_size,
        Some(&conf.solr.core),
        None,
    )
    .await
    .context("error ingesting data to solr")?;

    for r in reports.datasets.iter() {
        let time = humantime::format_duration(r.time);
        println!(
            "[{}]\n\
            sent    {}\n\
            skipped {}\n\
            took    {}\n\
        ",
            r.dataset.name(),
            r.sent,
            r.skipped,
            time,
        );
    }

    let time = humantime::format_duration(reports.total_time);
    println!("total time took {}", time);

    Ok(())
}
